#pragma once
/*
 *      Copyright (C) 2005-2013 Team XBMC
 *      http://www.xbmc.org
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with XBMC; see the file COPYING.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 */

#include "platform/util/StdString.h"

#include <boost/shared_ptr.hpp>

extern "C" {
#include <cmyth/cmyth.h>
};

template <class T> class MythPointer;

class MythEPGInfo
{
public:
  MythEPGInfo();
  MythEPGInfo(cmyth_epginfo_t cmyth_epginfo);

  bool IsNull() const;

  unsigned int ChannelID();
  CStdString ChannelName();
  CStdString Callsign();
  unsigned int SourceID();
  CStdString Title();
  CStdString Subtitle();
  CStdString Description();
  time_t StartTime();
  time_t EndTime();
  CStdString ProgramID();
  CStdString SeriesID();
  CStdString Category();
  CStdString CategoryType();
  unsigned int ChannelNumberInt();

private:
  boost::shared_ptr<MythPointer<cmyth_epginfo_t> > m_epginfo_t;
};
