#pragma once
/*
 *      Copyright (C) 2005-2012 Team XBMC
 *      http://www.xbmc.org
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with XBMC; see the file COPYING.  If not, see
 *  <http://www.gnu.org/licenses/>.
 *
 */

#include "platform/util/StdString.h"

#include <boost/shared_ptr.hpp>

extern "C" {
#include <cmyth/cmyth.h>
};

template <class T> class MythPointer;

class MythChannel
{
public:
  MythChannel();
  MythChannel(cmyth_channel_t cmyth_channel);

  bool IsNull() const;

  unsigned int ID();
  CStdString Name();
  unsigned int NumberInt();
  CStdString Number();
  CStdString Callsign();
  CStdString Icon();
  bool Visible();
  bool IsRadio();
  unsigned int SourceID();
  unsigned int MultiplexID();

private:
  boost::shared_ptr<MythPointer<cmyth_channel_t> > m_channel_t;
};
