#pragma once
/*
 *      Copyright (C) 2014 Fred Hoogduin
 *
 *  This Program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This Program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with XBMC; see the file COPYING.  If not, write to
 *  the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
 *  http://www.gnu.org/copyleft/gpl.html
 *
 */

#include "platform/threads/threads.h"

class CEventsThread : public PLATFORM::CThread
{
public:
  CEventsThread(void);
  ~CEventsThread(void);
  void Connect(void);
private:
  virtual void *Process(void);

  void HandleEvents(Json::Value events);

  bool m_subscribed;
  std::string m_monitorId;
};

